@@ -32,6 +32,26 @@
  * SECTION:soup-session
  * @short_description: Soup session state object
  *
+ * #SoupSession is the object that controls client-side HTTP. A
+ * #SoupSession encapsulates all of the state that libsoup is keeping
+ * on behalf of your program; cached HTTP connections, authentication
+ * information, etc.
+ *
+ * Most applications will only need a single #SoupSession; the primary
+ * reason you might need multiple sessions is if you need to have
+ * multiple independent authentication contexts. (Eg, you are
+ * connecting to a server and authenticating as two different users at
+ * different times; the easiest way to ensure that each #SoupMessage
+ * is sent with the authentication information you intended is to use
+ * one session for the first user, and a second session for the other
+ * user.)
+ *
+ * #SoupSession itself is an abstract class, with two subclasses. If
+ * you are using the glib main loop, you will generally want to use
+ * #SoupSessionAsync, which uses non-blocking I/O and callbacks. On
+ * the other hand, if your application is threaded and you want to do
+ * synchronous I/O in a separate thread from the UI, use
+ * #SoupSessionSync.
  **/
 
 typedef struct {
@@ -55,6 +75,8 @@ typedef struct {
 
 	SoupMessageQueue *queue;
 
+	char *user_agent;
+
 	SoupAuthManager *auth_manager;
 	SoupAuthManagerNTLM *ntlm_manager;
 
@@ -92,7 +114,9 @@ extern gboolean  soup_uri_host_equal (gconstpointer  v1,
 extern SoupURI  *soup_uri_copy_root  (SoupURI *uri);
 
 #define SOUP_SESSION_MAX_CONNS_DEFAULT 10
-#define SOUP_SESSION_MAX_CONNS_PER_HOST_DEFAULT 4
+#define SOUP_SESSION_MAX_CONNS_PER_HOST_DEFAULT 2
+
+#define SOUP_SESSION_USER_AGENT_BASE "libsoup/" PACKAGE_VERSION
 
 G_DEFINE_TYPE (SoupSession, soup_session, G_TYPE_OBJECT)
 
@@ -114,6 +138,7 @@ enum {
 	PROP_SSL_CA_FILE,
 	PROP_ASYNC_CONTEXT,
 	PROP_TIMEOUT,
+	PROP_USER_AGENT,
 
 	LAST_PROP
 };
@@ -191,6 +216,8 @@ finalize (GObject *object)
 	g_hash_table_destroy (priv->hosts);
 	g_hash_table_destroy (priv->conns);
 
+	g_free (priv->user_agent);
+
 	soup_auth_manager_free (priv->auth_manager);
 	if (priv->ntlm_manager)
 		soup_auth_manager_ntlm_free (priv->ntlm_manager);
@@ -296,7 +323,7 @@ soup_session_class_init (SoupSessionClass *session_class)
 				  "The maximum number of connections that the session can open at once",
 				  1,
 				  G_MAXINT,
-				  10,
+				  SOUP_SESSION_MAX_CONNS_DEFAULT,
 				  G_PARAM_READWRITE));
 	g_object_class_install_property (
 		object_class, PROP_MAX_CONNS_PER_HOST,
@@ -305,7 +332,7 @@ soup_session_class_init (SoupSessionClass *session_class)
 				  "The maximum number of connections that the session can open at once to a given host",
 				  1,
 				  G_MAXINT,
-				  2,
+				  SOUP_SESSION_MAX_CONNS_PER_HOST_DEFAULT,
 				  G_PARAM_READWRITE));
 	g_object_class_install_property (
 		object_class, PROP_USE_NTLM,
@@ -334,6 +361,41 @@ soup_session_class_init (SoupSessionClass *session_class)
 				   "Value in seconds to timeout a blocking I/O",
 				   0, G_MAXUINT, 0,
 				   G_PARAM_READWRITE));
+
+	/**
+	 * SoupSession:user-agent:
+	 *
+	 * If non-%NULL, the value to use for the "User-Agent" header
+	 * on #SoupMessage<!-- -->s sent from this session.
+	 *
+	 * RFC 2616 says: "The User-Agent request-header field
+	 * contains information about the user agent originating the
+	 * request. This is for statistical purposes, the tracing of
+	 * protocol violations, and automated recognition of user
+	 * agents for the sake of tailoring responses to avoid
+	 * particular user agent limitations. User agents SHOULD
+	 * include this field with requests."
+	 *
+	 * The User-Agent header contains a list of one or more
+	 * product tokens, separated by whitespace, with the most
+	 * significant product token coming first. The tokens must be
+	 * brief, ASCII, and mostly alphanumeric (although "-", "_",
+	 * and "." are also allowed), and may optionally include a "/"
+	 * followed by a version string. You may also put comments,
+	 * enclosed in parentheses, between or after the tokens.
+	 *
+	 * If you set a %user_agent property that has trailing
+	 * whitespace, #SoupSession will append its own product token
+	 * (eg, "<literal>libsoup/2.3.2</literal>") to the end of the
+	 * header for you.
+	 **/
+	g_object_class_install_property (
+		object_class, PROP_USER_AGENT,
+		g_param_spec_string (SOUP_SESSION_USER_AGENT,
+				     "User-Agent string",
+				     "User-Agent string",
+				     NULL,
+				     G_PARAM_READWRITE));
 }
 
 static gboolean
@@ -369,7 +431,7 @@ set_property (GObject *object, guint prop_id,
 	SoupURI *uri;
 	gboolean need_abort = FALSE;
 	gboolean ca_file_changed = FALSE;
-	const char *new_ca_file;
+	const char *new_ca_file, *user_agent;
 
 	switch (prop_id) {
 	case PROP_PROXY_URI:
@@ -433,6 +495,21 @@ set_property (GObject *object, guint prop_id,
 	case PROP_TIMEOUT:
 		priv->timeout = g_value_get_uint (value);
 		break;
+	case PROP_USER_AGENT:
+		g_free (priv->user_agent);
+		user_agent = g_value_get_string (value);
+		if (!user_agent)
+			priv->user_agent = NULL;
+		else if (!*user_agent) {
+			priv->user_agent =
+				g_strdup (SOUP_SESSION_USER_AGENT_BASE);
+		} else if (g_str_has_suffix (user_agent, " ")) {
+			priv->user_agent =
+				g_strdup_printf ("%s%s", user_agent,
+						 SOUP_SESSION_USER_AGENT_BASE);
+		} else
+			priv->user_agent = g_strdup (user_agent);
+		break;
 	default:
 		break;
 	}
@@ -467,6 +544,9 @@ get_property (GObject *object, guint prop_id,
 	case PROP_TIMEOUT:
 		g_value_set_uint (value, priv->timeout);
 		break;
+	case PROP_USER_AGENT:
+		g_value_set_string (value, priv->user_agent);
+		break;
 	default:
 		break;
 	}
@@ -563,12 +643,47 @@ redirect_handler (SoupMessage *msg, gpointer user_data)
 	const char *new_loc;
 	SoupURI *new_uri;
 
-	if (!SOUP_STATUS_IS_REDIRECTION (msg->status_code))
-		return;
-
 	new_loc = soup_message_headers_get (msg->response_headers, "Location");
-	if (!new_loc)
+	g_return_if_fail (new_loc != NULL);
+
+	if (msg->status_code == SOUP_STATUS_MOVED_PERMANENTLY ||
+	    msg->status_code == SOUP_STATUS_TEMPORARY_REDIRECT) {
+		/* Don't redirect non-safe methods */
+		if (msg->method != SOUP_METHOD_GET &&
+		    msg->method != SOUP_METHOD_HEAD &&
+		    msg->method != SOUP_METHOD_OPTIONS &&
+		    msg->method != SOUP_METHOD_PROPFIND)
+			return;
+	} else if (msg->status_code == SOUP_STATUS_SEE_OTHER ||
+		   msg->status_code == SOUP_STATUS_FOUND) {
+		/* Redirect using a GET */
+		g_object_set (msg,
+			      SOUP_MESSAGE_METHOD, SOUP_METHOD_GET,
+			      NULL);
+		soup_message_set_request (msg, NULL,
+					  SOUP_MEMORY_STATIC, NULL, 0);
+		soup_message_headers_set_encoding (msg->request_headers,
+						   SOUP_ENCODING_NONE);
+	} else {
+		/* Three possibilities:
+		 *
+		 *   1) This was a non-3xx response that happened to
+		 *      have a "Location" header
+		 *   2) It's a non-redirecty 3xx response (300, 304,
+		 *      305, 306)
+		 *   3) It's some newly-defined 3xx response (308+)
+		 *
+		 * We ignore all of these cases. In the first two,
+		 * redirecting would be explicitly wrong, and in the
+		 * last case, we have no clue if the 3xx response is
+		 * supposed to be redirecty or non-redirecty. Plus,
+		 * 2616 says unrecognized status codes should be
+		 * treated as the equivalent to the x00 code, and we
+		 * don't redirect on 300, so therefore we shouldn't
+		 * redirect on 308+ either.
+		 */
 		return;
+	}
 
 	/* Location is supposed to be an absolute URI, but some sites
 	 * are lame, so we use soup_uri_new_with_base().
@@ -592,6 +707,12 @@ connection_started_request (SoupConnection *conn, SoupMessage *msg,
 			    gpointer data)
 {
 	SoupSession *session = data;
+	SoupSessionPrivate *priv = SOUP_SESSION_GET_PRIVATE (session);
+
+	if (priv->user_agent) {
+		soup_message_headers_replace (msg->request_headers,
+					      "User-Agent", priv->user_agent);
+	}
 
 	g_signal_emit (session, signals[REQUEST_STARTED], 0,
 		       msg, soup_connection_get_socket (conn));
@@ -883,6 +1004,16 @@ queue_message (SoupSession *session, SoupMessage *msg,
 	soup_message_queue_append (priv->queue, msg);
 }
 
+/**
+ * SoupSessionCallback:
+ * @session: the session
+ * @msg: the message that has finished
+ * @user_data: the data passed to soup_session_queue_message
+ *
+ * Prototype for the callback passed to soup_session_queue_message(),
+ * qv.
+ **/
+
 /**
  * soup_session_queue_message:
  * @session: a #SoupSession
